
////////////////////// ULALA ///////////////////////
//
// Generación de video compatible con VGA
// tiempos del modo VESA 640x480
//
// Emulamos video del ZX spectrum: 
//
// 256x192 pixels con atributos de 8x8 pixels
// => 4 pixeles VGA por cada pixel Spectrum
//  512 pixelsVGA/línea, 384 líneas/luadro
// Memoria de sólo escritura en el lado de CPU

module video ( 
	input clk,		// clock input @25MHz
	input clkw,		// reloj de la CPU
	input [7:0]di,	// datos para escribir en la memoria
	input [12:0]aw,	// dirección de memoria para escribir
	input we,		// habilitacíon de escritura
	output hsyn,	// sincronismo horizontal
	output vsyn,	// sincronismo vertical
	output [3:0]video,	// Salida de video IGRB
	input [2:0]border	// color del borde (GRB)
);

//////////////////// Tiempos //////////////////////

reg [9:0]hc=0;	// Contador horizontal
reg hsyn=1;
wire hde;
reg hblk=0;

always @(posedge clk) begin
	if (hc==10'd799) hc<=0; else hc<=hc+1;
	if (hc==10'd591) hsyn<=0;
	if (hc==10'd687) hsyn<=1;
	if (hc==10'd575) hblk<=1;
	if (hc==10'd735) hblk<=0;
	
end

assign hde=~hc[9];

reg [9:0]vc=0;	// Contador vertical
reg vsyn=1;
reg vde=1;
reg vblk=0;

always @(negedge hblk) begin
	if (vc==10'd524) begin vc<=0; vde<=1; end else vc<=vc+1;
	if (vc==10'd441) vsyn<=0;
	if (vc==10'd443) vsyn<=1;
	if (vc==10'd383) vde<=0;
	if (vc==10'd431) vblk<=1;
	if (vc==10'd476) vblk<=0;
end

// Blank
wire blk;
assign blk = hblk|vblk;

// Data valid
wire dv;
assign dv=hde&vde;
// Memory read
wire rdv;
assign rdv = dv & (hc[3:0]==4'b0001);

//////////////////// Memoria de video //////////////////////

wire [7:0]vdata;

///// 6KB video memory /////
// 4KB
genram2p #(.AW(12),	.DW(8)) vram0 ( .clkr(clk),.clkw(clkw), .ao(vaddr[11:0]),
		  							.we(we4K),.di(di),.ai(aw[11:0]),.do(do4K));
// 2KB
genram2p #(.AW(11),	.DW(8)) vram4 ( .clkr(clk),.clkw(clkw), .ao(vaddr[10:0]),
		  							.we(we2K),.di(di),.ai(aw[10:0]),.do(do2K));
///// 1KB attribute memory /////
genram2p #(.AW(10),	.DW(8)) vram6 ( .clkr(clk),.clkw(clkw), .ao(ataddr[9:0]),
		  							.we(we1K),.di(di),.ai(aw[9:0]),.do(do1K));
wire [7:0]do4K;
wire [7:0]do2K;
wire [7:0]do1K;
wire we4K,we2K,we1K;
assign we4K=we & (~aw[12]);
assign we2K=we & aw[12] & (~aw[11]);
assign we1K=we & aw[12] & aw[11] & (~aw[10]);

// Direcciones de pixeles
wire [12:0]vaddr;
assign vaddr[12:0]={vc[8:7],vc[3:1],vc[6:4],hc[8:4]};

// Direcciones de atributos
wire [9:0]ataddr;
assign ataddr={vc[8:4],hc[8:4]};

//////////////////// Desplazador //////////////////////

reg [7:0]attr=0;	// attribute reg
reg [7:0]vshift;	// pixel shift reg
reg ddv=0;			// delayed data valid

always @(posedge clk) begin
	ddv<=dv;		// delayed data valid
	if (rdv) begin	// Read memory
		attr<=do1K; 
		vshift<=vaddr[12] ? do2K : do4K; 
	end else	// Shift
		if (hc[0]) vshift<={vshift[6:0],1'b0};
end

// Contador para parpadeo
reg [4:0]blink=0;
always @(posedge vsyn) blink<=blink+1;

// Salida de video
wire [3:0]pixel;
assign pixel=(vshift[7]^(attr[7]&blink[4])) ? {attr[6],attr[2:0]} : {attr[6],attr[5:3]};

assign video=(ddv)? pixel:(blk ? 4'b0000: {1'b0,border});

endmodule

//----------------------------------------------------------------------------
//-- Memoria RAM de 2 puertos genérica
//----------------------------------------------------------------------------

module genram2p (        //-- 2 Puertos
    input clkr,         // reloj de lectura
    input clkw,			// reloj de escritura 
    input wire [AW-1: 0] ao,      //-- Direcciones
    input wire [AW-1: 0] ai,      //-- Direcciones
    input wire we,                  //-- Escritura si 1
    input wire [DW-1: 0] di,   //-- Dato de entrada
    output reg [DW-1: 0] do   //-- Dato a escribir
);
//-- Parametro: Nombre del fichero con el contenido de la RAM
//parameter INITFILE = "video.hex";
parameter AW = 13;
parameter DW = 8;

  //-- Memoria
  reg [DW-1: 0] ram [0: (1<<AW)-1];

  //-- Lectura de la memoria
  always @(posedge clkr) begin
    do <= ram[ao];
  end
  //-- Esctura de la memoria
  always @(posedge clkw) begin
    if (we) ram[ai] <= di;
  end

//-- Cargar en la memoria el fichero ROMFILE
//-- Los valores deben estan dados en hexadecimal
//initial begin
//  $readmemh(INITFILE, ram);
//end

endmodule

